@@ -7,6 +7,7 @@ Example:
 
 # Stdlib imports
 import logging
+from typing import Dict, Optional
 
 # Third-party imports
 from flask import Flask
@@ -21,6 +22,7 @@ logger = logging.getLogger(__name__)
 
 # In order to be able to generate API docs automatically, it is unfortunately
 # necessary to wrap the preparatory code in a try:except: statement.
+conf: Optional[Dict]
 try:
     conf = config.get_validated_config()
 except FileNotFoundError:
@@ -39,7 +41,7 @@ if conf:
     if __name__ == "__main__":  # pragma: no cover
         logger.warning(
             "Starting Flask debug server.\nIn production, please use a WSGI server, "
-            + "e.g. 'gunicorn -w 4 -b 127.0.0.1:5000 mllaunchpad.wsgi:application'"
+            "e.g. 'gunicorn -w 4 -b 127.0.0.1:5000 mllaunchpad.wsgi:application'"
         )
         # Flask apps must not be run in debug mode in production, because this allows for arbitrary code execution.
         # We know that and advise the user that this is only for debugging, so this is not a security issue (marked nosec):
